import 'package:consoledoctor_common/consoledoctor_common.dart';
import 'package:dio/dio.dart';
import 'package:logging/logging.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:version_checker/src/remote_version_response.dart';
import 'package:version_checker/src/version_validation_response.dart';

abstract class VersionChecker {
  late final Map<String, String> headers;
  final Dio dio = Dio();
  final String baseUri;
  late final Logger logger;

  VersionChecker({required this.baseUri, required String loggerName}) {
    logger = Logger(loggerName);

    dio.options.connectTimeout = const Duration(seconds: 15);
    dio.options.receiveTimeout = const Duration(seconds: 15);
    dio.options.headers = buildHeaders();
    dio.options.baseUrl = baseUri.endsWith("/") ? baseUri : "$baseUri/";
  }

  Map<String, dynamic> buildHeaders();

  Future<DataResponse<RemoteVersionResponse>> getLatestVersionInfo();

  Future<PackageInfo> getCurrentVersionInfo() async => await PackageInfo.fromPlatform();

  Future<VersionValidationResponse> validateAppVersion() async {
    try {
      var current = await getCurrentVersionInfo();
      var latest = await getLatestVersionInfo();

      if (!latest.success) {
        logger.severe('Error acquiring latest version info. Message: ${latest.message}. Exception: ${latest.exceptionMessage}');
      }

      return VersionValidationResponse(latest.success, latest.message, current.version, latest.result?.version ?? "", latest.result?.downloadUrl ?? "");
    } catch(e) {
      return VersionValidationResponse(false, "An error occurred when trying to validate the app version", "", "", "");
    }
  }
}